#!/usr/bin/perl -w
################################################################################
# fan_backup.pl : backup FAN server datas to on directory.
# Copyright (C) 2009 FAN Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
################################################################################
# Description : backup FAN server datas to on directory.
################################################################################
# Version : 0.1
################################################################################
# Author : Cedric TEMPLE <cedrictemple@cedrictemple.info>
################################################################################

use strict;

use vars qw($mysql_user $mysql_passwd $mysql_host $mysql_database_oreon $mysql_database_ods $mysql_database_ndo);

require '/etc/centreon/conf.pm';

my $current_debug_level = 3;
my $backup_dir = '/var/backup';
my $mysql_database_ndo = 'nagios';
#my $mysql_database_nareto = 'nareto';

# my $mysql_host;
# my $mysql_user;
# my $mysql_passwd;
# my $mysql_database_oreon;
# my $mysql_database_ods;

# backup database $1 to directory $2. Use user $3 and password $4 to connect
# password must be set to '' to something
sub backup_database($$$$);

# backup directory source $1 to directory destination $2.
sub backup_directory($$);

# log message $1 in a log file if level $2 >= current debug level
sub debug($$);


# main
my $date = `date +%Y%m%d_%Hh%Mm%Ss`;
$date =~ s/\s+//g;
$backup_dir .= '/'.$date;

backup_database($mysql_database_oreon, $backup_dir.'/databases/'.$mysql_database_oreon, $mysql_user, $mysql_passwd);
backup_database($mysql_database_ods, $backup_dir.'/databases/'.$mysql_database_ods, $mysql_user, $mysql_passwd);
backup_database($mysql_database_ndo, $backup_dir.'/databases/'.$mysql_database_ndo, $mysql_user, $mysql_passwd);
#backup_database($mysql_database_nareto, $backup_dir.'/databases/'.$mysql_database_nareto, $mysql_user, $mysql_passwd);

backup_directory( '/usr/share/nagios/images/logos/', $backup_dir.'/logos' );
backup_directory( '/usr/lib/nagios/plugins/', $backup_dir.'/plugins' );
backup_directory( '/var/www/dokuwiki/data/pages/', $backup_dir.'/wiki' );

# backup database $1 to directory $2. Use user $3 and password $4 to connect
# password must be set to '' to something
sub backup_database($$$$){
    my ($db, $dir, $user, $pass) = ($_[0], $_[1], $_[2], $_[3]);
    my $error_message;
    my $return_code;

    debug( 'backup database '.$_[0] . ' to directory '. $_[1].' using user '.$_[2].' and password '.$_[3], 2);

    if ( ! -d $dir ) {
	$error_message = `mkdir -p $dir 2>&1`;
	$return_code = $?;
	if ( $return_code != 0 ) {
	    debug( 'ERROR! Could NOT create directory '.$dir.' Error : '.$error_message, 0);
	}
    }
    else {
	debug( "Directory $dir exists. I do not have to create it!", 2 );
    }
    
    my $mysqldump_args = "--opt $db --user=$user ";
    
    if ( defined($pass) && ($pass ne "")) {
	$mysqldump_args .= " --password=$pass > $dir/$db.sql 2> /dev/null";
    }
    else {
	$mysqldump_args .= " > $dir/$db.sql 2> /dev/null";
    }
    debug( 'launch mysqldump '.$mysqldump_args, 2);
    `mysqldump $mysqldump_args`;
    $return_code = $?;
    if ( $return_code != 0 ) {
	debug( 'ERROR! Could NOT backup database '.$db, 0);
    }

    
    debug( "gzip -9 $dir/$db.sql", 2);
    $error_message = `gzip -9 $dir/${db}.sql 2>&1`;
    $return_code = $?;
    if ( $return_code != 0 ) {
        debug( 'ERROR! Could NOT compress file '.$dir."/".$db.".sql".' Error message :'.$error_message, 0);
    }

}


# log message $1 in a log file if level $2 <= current debug level
sub debug($$){
    if ( $_[1] <= $current_debug_level ) {
	open( DEBUG_FILE, ">> /var/log/fan_backup") or die ("Could not open log file /var/log/fan_backup");
	printf DEBUG_FILE "DEBUG %d : %s\n", $_[1], $_[0];
	close ( DEBUG_FILE ) ;
    }

}



# backup directory source $1 to directory destination $2.
sub backup_directory($$){
    my ($source, $destination) = ($_[0], $_[1]);
    my $error_message;
    my $return_code;

    debug( 'backup directory source '.$_[0] . ' to directory destination '. $_[1], 2);

    if ( ! -d $destination ) {
	$error_message = `mkdir -p $destination 2>&1`;
	$return_code = $?;
	if ( $return_code != 0 ) {
	    debug( 'ERROR! Could NOT create directory '.$destination.' Error : '.$error_message, 0);
	}
    }
    else {
	debug( "Directory $destination exists. I do not have to create it!", 2 );
    }
    
    debug( "cp -a $source $destination", 2);
    $error_message = `cp -a $source $destination 2>&1`;
    $return_code = $?;
    if ( $return_code != 0 ) {
        debug( 'ERROR! Could NOT copy directory'. $source.' to directory '. $destination.'. Error message :'.$error_message, 0);
    }
    
}
